// This example shows how a UDP-UDP or a xUDP-xUDP connection can be build
// without the use or intervention of a Stack-Mapper program.
// When a pure xUDP/UDP network is build with UDP/xUDP-NetStacks and
// the communication is restricted to NetStack<>NetStack or between
// a NetStack and a PC which can handle UDP or xUDP packets a
// Stackmapper is not necessary and the program size and much more
// the RAM memory useage is reduced. The 3 necessary Buffers
// can be resized to a minimum (<100bytes) depending of the
// desired data block sizes.
// If a PC is involved a program on the PC must be provided which
// can handle UDP and/or xUDP packets. Such a utility is the included
// "xUDP Check" program. Please note that there is no information
// within a packet whether it is an UDP or xUDP type of packet.
// So the best way is to use only UDP or xUDP, but no mix-up.
// For security reasons the xUDP protocol should be prefered.
program xUDP_xUDP;

Device = mega128, VCC=5;

Import SysTick, Netstack, SerPort;

From System import ;

From NetStack import RTL8019;

Define
        XData       = $8000,$FFFF;
        XData1      = $4300,$4FFF, noInit;  // I/O area of RTL8019
        ProcClock   = 16000000;             // 16MHz
        SysTick     = 2;                    // 2msec
        StackSize   = $40, iData;           // at least 50 bytes stacksize
        FrameSize   = $50, iData;           // at least 64 bytes frame size
        NetStack    = 1536, xData;          // total buffer size, memory location
        ARPcacheSize= 20;                   // arpcache entries
        RTL8019     = $4300, Int5;          // RTL base address, Interruptpin
//        RTLreset    = PortD, 2;             // optional reset line to RTL
        SerPort     = 57600;
        RxBuffer    = 80, iData;
        TxBuffer    = 80, iData;


Implementation

{$IDATA}
{--------------------------------------------------------------}
{ Type Declarations }
type

{--------------------------------------------------------------}
{ Const Declarations }
const
  // these 2 constants !must! be provided by the user
  Date     : string   = 'dd.mm.yyyy';
  Time     : string   = 'hh:mm';

{$EEPROM}
StructConst
  eDummy            : word        = 0;                // first 2 bytes in AVR EEprom not useable
  // these 7 constants !must! be provided by the user
  Net_ARPTimeOut    : Byte        = 100;              // Systicks !!!
  Net_ARPRetry      : Byte        =  3;               // Request retries
  Net_SendTimeOut   : Byte        = 250;              // Systicks !!!  = 500msec
  Net_SendRetry     : Byte        = 1;                // Send retries    1 times
  Net_UDPAknPort    : Word        = 1189;             // Handshake port
  Net_LocalMACaddr  : tMACaddress = ($02, $2C, $2C, $CE, $AE, $16);
  Net_LocalIPaddr   : tIPaddress  = (192, 168, 1, 16);
  Net_LocalMask     : tIPaddress  = (255, 255, 255, 0);
  Net_xUDPMapper    : tIPaddress  = (192, 168, 1, 2); // not really used for UDP/xUDP
  Net_DefaultGateWay: tIPaddress  = (0, 0, 0, 0);

{ these 2 constant are only necessary if a xUDP-Mapper is involved
  Net_BoardName     : String[20]  = 'E-LAB xUDP Stack 1';
  Net_FirmWare      : String[20]  = 'E-LAB Telnet Test';
}

{--------------------------------------------------------------}
{ Var Declarations }
{$PDATA}
var
  LED3[@PortE, 6]             : bit;
  LED4[@PortE, 7]             : bit;

{$IDATA}
VAR
  PacketInfo            : TGetPacketInfo;               // info for received packet
  UDPpackInfo           : tUDPpackInfo;                 // info for transmitted UDP packet
  myBuffer              : array[0..255] of byte;        // local data buffer (transmit)
  myString[@myBuffer]   : string[255];                  //   "        "         "
  Ptr                   : pointer;                      //
  count                 : word;
  NetError              : TNetErrorType;
  ch                    : char;
  st                    : string[32];
  st1[@st]              : string[20];
  bb                    : byte;
  Timer1                : SysTimer;
  run                   : boolean;
  speed                 : boolean;


{$IDATA}
{--------------------------------------------------------------}
// functions
procedure SignOn;
begin
  // list serial commands
  writeln(SerOut);
  writeln(SerOut, 'UDP-UDP and xUDP-xUDP demo');
  writeln(SerOut, 'b = bulk 1000 packets');
  writeln(SerOut, 'r = run or stop');
  writeln(SerOut, 's = send an UDP packet');
  writeln(SerOut, 'u = set standard UDP mode');
  writeln(SerOut, 'x = set extended UDP mode');
end;

// example how to build an UDP or xUDP packet
// This is different to all other packet types !!!
Function SendStr(IPAD : TIPAddress; port : word; ST : String[40]) : Boolean;
VAR
  OK : Boolean;
Begin
  UDPpackInfo.TargetIPAddr:= IPAD;
  UDPpackInfo.SourcePort  := 1889;
  UDPpackInfo.DestPort    := port;
  UDPpackInfo.DataPtr     := @ST+1;
  UDPpackInfo.DataLen     := word(length(ST));
  OK:= Net_SendUDPPacket(UDPpackInfo) = NetErrNone;
  if not Ok then
    writeln(SerOut, 'tx failed');
  endif;
  Return(OK);
end;


// This function is optional. If provided the NetStack enters
// this function with the LED info
procedure NetStack_LEDs(LED : byte; OnOff : boolean);
begin
  case LED of
    1 : LED3:= not OnOff; |   // indicates a Rx-packet was processed by the NIC/RTL8019
    2 : LED3:= not OnOff; |   // indicates that an ident request was processed
    3 : LED4:= not OnOff; |   // indicates that Tx-packet was processed
  endcase;
end;

{--------------------------------------------------------------}
{ Main Program }
begin
  // LED port
  DDRE:= DDRE or $C0;
  // test LEDs
  LED3:= false;
  Mdelay(200);
  LED4:= false;
  Mdelay(200);
  LED3:= true;
  Mdelay(200);
  LED4:= true;

  SetSysTimer(Timer1, 100);                                        // 100 ticks = 1sec

  // enable NetStack for receiving packets
  Net_StackSetActive(TRUE);
  // allow UDP port accesses
  Net_AddUsrPort(8090, 1);
  Net_AddUsrPort(8890, 2);
  Net_AddUsrPort(1736, 3);

  // switch from standard UDP to our Extended-UDP
  Net_ExtendedUDP(TRUE);                                 // enable extended UDP

  EnableInts;
  // prepare to get ready
  Net_StartNet;
  // necessary for mega128
  EIMSK:= 32;                                            // INT5 Enable
  EICR:= 0;                                              // 0;

  SignOn;
  SendStr(Net_xUDPMapper, 8890, '1sec Message '+IntToStr(count));

  loop
    // look for a incoming packet
    if Net_PollPacket then
      // fetch the packet from NetStack
      IF Net_GetPacket(PacketInfo) then
        // analyze the packet type
        Case PacketInfo.TypeOfPacket of
          ptTCP         : {TCP-Packet not implemented}
                        |
          ptUDP         : if PacketInfo.UserPortID in[1, 2, 3] then
                            // with our UDP/xUDP we can receive packets via these ports
                            // which where introduced by "Net_AddUsrPort"
                            LED3:= false;
                            LED4:= false;
                            WriteLn(SerOut);
                            Write(SerOut, 'Port: ');
                            if PacketInfo.UserPortID = 1 then                           // port = 8090
                              WriteLn(SerOut, '8090');
                            elsif PacketInfo.UserPortID = 2 then                        // port = 8890
                              WriteLn(SerOut, '8890');
                            else                                                        // port = 1736
                              WriteLn(SerOut, '1736');
                            endif;
                            WriteLn(SerOut, 'bytes: '+IntToStr(_RX_DATA_COUNT));
                            ptr:= Net_RxInitDatablock(idxUDP);
                            while Net_RxGetDataStr(st) do
                              WriteLn(SerOut, '  '+st);
                            endwhile;
                            LED3:= true;
                            LED4:= true;
                          // not used here, only for information
                          elsif PacketInfo.UserPortID = 201 then
                            // this is a StackMapper info message, not imported
                            case MAPPERPACKTYPE of
                              idxEmail  :
                                        |
                              idxTelNet :
                                        |
                              idxFTP    :
                                        |
                              idxFile   :
                                        |
                              idxChat   :
                                        |
                              idxHTTP   :
                                        |
                            endcase;
                          endif;
                        |
          ptFTP         : {FTP-Packet not implemented}
                        |
          ptTelnet      : {Telnet-Packet not implemented}
                        |
          ptHTTP        : {HTTP-Packet not implemented}
                        |
        else
          WriteLn(SerOut, 'any 1');
        endcase;
      else
        WriteLn(SerOut, 'any 2');
      endif;
    endif;
    if SerStat then
      ch:= SerInp;
      case ch of
        // the following commands are used only for debug purpose
        '!'   : SignOn;
                |
        'b'   : run:= false;
                speed:= not speed;
                count:= 0;
                |
        'r'   : run:= not run;
                count:= 0;
                speed:= false;
                |
        's'   : // send an UDP packet. We use "Net_xUDPMapper" IP because this is our target PC
                // but remember there is no Mapper present for simple UDP/xUDP
                SendStr(Net_xUDPMapper, 8890, 'this is the UDP packet '+IntToStr(count));
                |
        'u'   : // set standard UDP mode
                Net_ExtendedUDP(false);                                // disable extended UDP
                |
        'x'   : // set extended (xUDP) mode
                Net_ExtendedUDP(TRUE);                                 // enable extended UDP
                |
      else
        SerOut('?');
      endcase;
    endif;
    if isSysTimerZero(Timer1) and run then
      SetSysTimer(Timer1, 500);                                        // 500 ticks = 1sec
      inc(count);
      SendStr(Net_xUDPMapper, 8890, '1sec Message '+IntToStr(count));
      WriteLn(SerOut, IntToStr(count));
    elsif speed then
      if count < 1000 then
        inc(count);
        SendStr(Net_xUDPMapper, 8890, 'Message '+IntToStr(count));
        // because the NetStack + AVR is much faster then the PC
        // we need a short delay for best throughput results
        mDelay(1);
      else
        count:= 0;
        speed:= false;
         WriteLn(SerOut, 'done');
      endif;
    endif;
  endloop;
end xUDP_xUDP.


